@@ -1,7 +1,7 @@
 #
 # These Python module have been developed by V.A. Sole, from the European
 # Synchrotron Radiation Facility (ESRF) to build a frozen version of PyMca.
-# Given the nature of this work, these module can be considered public domain. 
+# Given the nature of this work, these module can be considered public domain.
 # Therefore redistribution and use in source and binary forms, with or without
 # modification, are permitted provided the following disclaimer is accepted:
 #
@@ -46,7 +46,7 @@ def load_PyQt4_Qt(finder, module):
        to this technique over pure Python; ignore the absence of some of
        the modules since not every installation includes all of them."""
     try:
-        #This modules does not seem to be always present 
+        #This modules does not seem to be always present
         finder.IncludeModule("PyQt4._qt")
     except ImportError:
         pass
@@ -63,59 +63,34 @@ def load_PyQt4_Qt(finder, module):
 
 _hooks.load_PyQt4_Qt = load_PyQt4_Qt
 
-if 0:
-    #This does not seem to solve the OpenGL problem
-    def _load_OpenGL(finder, module):
-        """
-        OpenGL >= 3.0.0 is ctypes and setuptools based. Therefore, a plethora of
-        modules are missed by the finder. So force inclusion of them all.
-        """
-        import OpenGL
-        version = -1
-        try:
-            version = int(OpenGL.version.__version__.split(".")[0])
-        except:
-            pass
-        if version >= 3:
-            basedir, sep, basemod = module.path[0].rpartition(os.sep)
-        for root, dirs, files in os.walk(module.path[0]):
-            package = root.replace(basedir, "", 1).strip(sep).replace(sep, ".")
-        if package != "OpenGL.tests": # ignore the OpenGL.tests package
-            finder.IncludePackage(package)
-    _hooks.load_OpenGL = _load_OpenGL
-
 
 PyMcaInstallationDir = "build"
 if sys.platform != "windows":
-    PyMcaDir = os.path.join(PyMcaInstallationDir, "PyMca").replace(" ","_")
+    PyMcaDir = os.path.join(PyMcaInstallationDir, "PyMca5").replace(" ","_")
 else:
-    PyMcaDir =os.path.join(PyMcaInstallationDir, "PyMca")
+    PyMcaDir =os.path.join(PyMcaInstallationDir, "PyMca5")
+
 #make sure PyMca is freshly built
 if sys.platform == 'win32':
     if MINGW:
         # MinGW compiler needs two steps
-        cmd = "python setup.py build -c mingw32"
+        cmd = "python setup.py build -c mingw32 --distutils"
         if os.system(cmd):
             print("Error building PyMca")
             sys.exit(1)
 
-cmd = "python setup.py install --install-lib %s" % PyMcaInstallationDir
+cmd = "python setup.py install --install-lib %s --distutils" % PyMcaInstallationDir
 if os.system(cmd):
     print("Error building PyMca")
     sys.exit(1)
 
+
+# PyMca expected to be properly installed
 include_files = []
 
 # this is critical for Qt to find image format plugins
 include_files.append(("qtconffile", "qt.conf"))
 
-# PyMca data
-include_files.append(("changelog.txt", os.path.join("PyMcaData", "changelog.txt")))
-include_files.append((os.path.join("PyMca", "PyMcaData"), "PyMcaData"))
-
-# put the PyMca plugins as "data" to allow the user to add his own plugins
-include_files.append((os.path.join("PyMca", "PyMcaPlugins"), "PyMcaPlugins"))
-
 # Add the qt plugins directory
 import PyQt4.Qt as qt
 app = qt.QApplication([])
@@ -126,7 +101,9 @@ for pluginSet in glob.glob(os.path.join(pluginsDir,'*')):
         if sys.platform == 'win32':
             ext = "*dll"
         else:
-            #for darwin platfrom I use py2app
+            if sys.platform.startswith("darwin"):
+                print("WARNING: Not ready for this platform")
+            #for darwin platform I use py2app
             #this only applies to linux
             ext = "*so"
         destination = os.path.join("plugins", plugin)
@@ -136,20 +113,12 @@ for pluginSet in glob.glob(os.path.join(pluginsDir,'*')):
                                  os.path.join(destination,os.path.basename(f))))
 
 #I should use somehow absolute import ...
-sys.path = [PyMcaDir, os.path.dirname(PyMcaDir)] + sys.path
+sys.path = [PyMcaInstallationDir] + sys.path[1:]
 try:
     import ctypes
     import OpenGL
-    import Object3D
+    from PyMca5 import Object3D
     OBJECT3D = True
-    try:
-        #ESRF very special distribution ...
-        import Object3DCTools
-        import Object3DQhull
-        OBJECT3DCTOOLS = True
-    except:
-        #fine, it is inside Object3D
-        OBJECT3DCTOOLS = False
 except:
     OBJECT3D = False
 
@@ -164,8 +133,6 @@ except:
 # For the time being I leave SciPy out
 SCIPY = False
 
-import PyMcaMain
-import PyMcaPlugins
 try:
     import matplotlib
     MATPLOTLIB = True
@@ -175,7 +142,7 @@ except ImportError:
 try:
     import pyopencl
     OPENCL = True
-    import sift
+    from PyMca5.PyMcaMath import sift
 except :
     OPENCL = False
 
@@ -201,85 +168,101 @@ try:
                     'h5py._conv', 'h5py._proxy']
     else:
         H5PY_SPECIAL = True
-        includes = []        
+        includes = []
 except:
     includes = []
 
+try:
+    import fisx
+    FISX = True
+except ImportError:
+    print("Please install fisx module prior to freeze PyMca")
+    FISX = False
+    raise
+
+try:
+    import hdf5plugin
+except ImportError:
+    pass
+
+
 #some standard encodings
 includes.append('encodings.ascii')
 includes.append('encodings.utf_8')
 includes.append('encodings.latin_1')
+import PyMca5
+
+special_modules = [os.path.dirname(PyMca5.__file__),
+                  os.path.dirname(ctypes.__file__)]
+
+if sys.platform == "win32":
+    try:
+        import hdf5plugin
+        special_modules.append(os.path.dirname(hdf5plugin.__file__))
+    except ImportError:
+        print("Please install hdf5plugin prior to freeze PyMca")
+        raise
+
+
+excludes = ["Tkinter", "tkinter",
+            'tcl','_tkagg', 'Tkconstants',
+            "scipy", "Numeric", "numarray"]
+
+try:
+    import IPython
+    if IPython.__version__.startswith('2'):
+        special_modules.append(os.path.dirname(IPython.__file__))
+        includes.append("colorsys")
+        import pygments
+        special_modules.append(os.path.dirname(pygments.__file__))
+        import zmq
+        special_modules.append(os.path.dirname(zmq.__file__))
+        import pygments
+        #includes.append("IPython")
+        #includes.append("IPython.qt")
+        #includes.append("IPython.qt.console")
+        #includes.append("IPython.qt.console.rich_ipython_widget")
+        #includes.append("IPython.qt.inprocess")
+        #includes.append("IPython.lib")
+except ImportError:
+    print("Console plugin not available")
+    pass
+
+if sys.version < '3.0':
+    #https://bitbucket.org/anthony_tuininga/cx_freeze/issues/127/collectionssys-error
+    excludes.append("collections.abc")
+
+if H5PY_SPECIAL:
+    special_modules.append(os.path.dirname(h5py.__file__))
+if OPENCL:
+    special_modules.append(os.path.dirname(pyopencl.__file__))
+    includes.append("pytools")
+    includes.append("decorator")
+else:
+    excludes.append("pyopencl")
+if MDP:
+    #mdp versions above 2.5 need special treatment
+    if mdp.__version__  > '2.5':
+        special_modules.append(os.path.dirname(mdp.__file__))
+if MATPLOTLIB:
+    special_modules.append(os.path.dirname(matplotlib.__file__))
+if SCIPY:
+    special_modules.append(os.path.dirname(scipy.__file__))
 
-#make sure all PyMca modules are there
-for module in glob.glob(os.path.join('PyMca', '*.py')):
-    m = "PyMca."+os.path.basename(module)[:-3]
-    if DEBUG:
-        print("Adding %s" % m)
-    includes.append(m)
-    
 if OBJECT3D:
     includes.append("logging")
-    excludes = ["OpenGL", "Tkinter", "PyMca.Object3D", "Object3D",
-                "PyMca.PyMcaPlugins", "PyMcaPlugins",
-                "scipy", "Numeric", "numarray"] 
-    #This requieres the use of the environmental variable MATPLOTLIBDATA
-    #pointing to mpl-data directory to work
-    special_modules =[os.path.dirname(ctypes.__file__),
-                  os.path.dirname(OpenGL.__file__),
-                  os.path.dirname(Object3D.__file__)]
-    if H5PY_SPECIAL:
-        special_modules.append(os.path.dirname(h5py.__file__))
-    if OPENCL:
-        special_modules.append(os.path.dirname(pyopencl.__file__))
-        excludes.append("PyMca.sift")
-        special_modules.append(os.path.dirname(sift.__file__))
-    else:
-        excludes.append("pyopencl")
-    if MDP:
-        #mdp versions above 2.5 need special treatment
-        if mdp.__version__  > '2.5':
-            special_modules.append(os.path.dirname(mdp.__file__))
-    if MATPLOTLIB:
-        special_modules.append(os.path.dirname(matplotlib.__file__))
-    if SCIPY:
-        special_modules.append(os.path.dirname(scipy.__file__))
-    for f in special_modules:
-            include_files.append((f,os.path.basename(f)))
-    if OBJECT3DCTOOLS:
-        excludes.append("Object3DCTools")
-        excludes.append("Object3DQhull")
-        for f in [Object3DCTools.__file__,Object3DQhull.__file__]: 
-            o3ddir = os.path.dirname(Object3D.__file__)
-            include_files.append((f, 
-                            os.path.join(os.path.basename(o3ddir), os.path.basename(f))))
-else:
-    excludes = ["Tkinter", "PyMca.PyMcaPlugins",
-                "PyMcaPlugins", "scipy", "Numeric", "numarray"]
-    special_modules = [os.path.dirname(ctypes.__file__)]
-    if H5PY_SPECIAL:
-        special_modules.append(os.path.dirname(h5py.__file__))
-    if OPENCL:
-        excludes.append("PyMca.sift")
-        special_modules.append(os.path.dirname(sift.__file__))
-        special_modules.append(os.path.dirname(pyopencl.__file__))
-    else:
-        excludes.append("pyopencl")
-    if MDP:
-        #mdp versions above 2.5 need special treatment
-        if mdp.__version__  > '2.5':
-            special_modules.append(os.path.dirname(mdp.__file__))
-    if MATPLOTLIB:
-        special_modules.append(os.path.dirname(matplotlib.__file__))
-    if SCIPY:
-        special_modules.append(os.path.dirname(scipy.__file__))
-    for f in special_modules:
-            include_files.append((f,os.path.basename(f)))
-    
+    excludes.append("OpenGL")
+    special_modules.append(os.path.dirname(OpenGL.__file__))
+
+if FISX:
+    special_modules.append(os.path.dirname(fisx.__file__))
+
+for f in special_modules:
+        include_files.append((f,os.path.basename(f)))
+
 for f in ['qt', 'qttable', 'qtcanvas', 'Qwt5']:
     excludes.append(f)
 
-#Next line was for the plugins in frozen but now is in shared zip library
-#include_files.append((PyMcaDir, "PyMca"))
 buildOptions = dict(
         compressed = True,
         include_files = include_files,
@@ -291,7 +274,8 @@ buildOptions = dict(
         #includes = ["Object3D"],
         #path = [PyMcaDir] + sys.path
         )
-install_dir = PyMcaDir + " " + PyMcaMain.__version__
+
+install_dir = PyMcaDir + " " + PyMca5.version()
 if not sys.platform.startswith('win'):
     install_dir = install_dir.replace(" ","")
 if os.path.exists(install_dir):
@@ -313,7 +297,7 @@ if os.path.exists(install_dir):
     except:
         print("Unexpected error:", sys.exc_info())
         pass
-        
+
 if os.path.exists('bin'):
     for f in glob.glob(os.path.join('bin','*')):
         os.remove(f)
@@ -322,36 +306,46 @@ installOptions = dict(
     install_dir= install_dir,
 )
 
-exec_list = ["PyMcaMain",
-             "PyMcaBatch",
-             "QStackWidget",
-             "PeakIdentifier",
-             "EdfFileSimpleViewer",
-             "PyMcaPostBatch",
-             "Mca2Edf",
-             "ElementsInfo"]
 
-for f in exec_list:
+exec_list = {"PyMcaMain": os.path.join(PyMcaDir, "PyMcaGui", \
+                                       "pymca", "PyMcaMain.py"),
+             "PyMcaBatch": os.path.join(PyMcaDir, "PyMcaGui", \
+                                        "pymca", "PyMcaBatch.py"),
+             "QStackWidget":os.path.join(PyMcaDir, "PyMcaGui", \
+                                         "pymca", "QStackWidget.py"),
+             "PeakIdentifier":os.path.join(PyMcaDir, "PyMcaGui", \
+                                        "physics", "xrf", "PeakIdentifier.py"),
+             "EdfFileSimpleViewer": os.path.join(PyMcaDir, "PyMcaGui", \
+                                                 "pymca", "EdfFileSimpleViewer.py"),
+             "PyMcaPostBatch": os.path.join(PyMcaDir, "PyMcaGui", \
+                                            "pymca", "PyMcaPostBatch.py"),
+             "Mca2Edf": os.path.join(PyMcaDir, "PyMcaGui", \
+                                     "pymca", "Mca2Edf.py"),
+             "ElementsInfo":os.path.join(PyMcaDir, "PyMcaGui", \
+                                         "physics", "xrf", "ElementsInfo.py"),
+             }
+
+for f in list(exec_list.keys()):
     executable = os.path.join(install_dir, f)
     if os.path.exists(executable):
         os.remove(executable)
 
-         
+
 executables = []
-for python_module in exec_list:
+for key in exec_list:
     icon = None
     # this allows to map a different icon to each executable
     if sys.platform.startswith('win'):
-        if python_module in ["PyMcaMain", "QStackWidget"]:
+        if key in ["PyMcaMain", "QStackWidget"]:
             icon = os.path.join(os.path.dirname(__file__), "icons", "PyMca.ico")
-    executables.append(Executable(os.path.join(PyMcaDir, python_module+".py"),
+    python_module = exec_list[key]
+    executables.append(Executable(python_module,
                                   icon=icon))
 
-
 setup(
-        name = "PyMca",
-        version = PyMcaMain.__version__,
-        description = "PyMca %s" % PyMcaMain.__version__,
+        name = "PyMca5",
+        version = PyMca5.version(),
+        description = "PyMca %s" % PyMca5.version(),
         options = dict(build_exe = buildOptions,
                        install_exe = installOptions
                        ),
@@ -422,45 +416,43 @@ if not sys.platform.startswith('win'):
                                        os.path.join(install_dir, f.lower())))
             if f == "PyMcaPostBatch":
                 os.system("cp -f %s %s" % (executable, os.path.join(install_dir, 'rgbcorrelator')))
-            
+
 
 #cleanup
-for f in glob.glob(os.path.join(os.path.dirname(__file__),"PyMca", "*.pyc")):
+for f in glob.glob(os.path.join(os.path.dirname(__file__),"PyMca5", "*.pyc")):
     os.remove(f)
 
 if not sys.platform.startswith('win'):
     #Unix binary ...
-    readline = 'libreadline.so.4'
-    for dirname in ['/lib','/usr/lib']:
-        fname = os.path.join(dirname, readline)
-        if os.path.exists(fname):
-            cmd =  "cp -f %s %s" % (fname, os.path.join(install_dir, readline))
-            os.system(cmd)
-            if dirname == '/lib':
-                #readline is from suse82 systems at ESRF
-                #and at a certain point I had to add these two libraries
-                fname = '/usr/lib/libg2c.so.0'
-                cmd = "cp -f %s %s" % (fname,
-                                       os.path.join(install_dir, 'libg2c.so.0'))
+    for dirname in ['/lib','/usr/lib', '/usr/X11R6/lib/']:
+        for fname0 in ["libreadline.so.4",
+                       "libgthread-2.0.so.0",
+                       "libglib-2.0.so.0",
+                       "libpng12.so.0",
+                       "libfreetype.so.6",
+                       "libXrender.so.1",
+                       "libXxf86vm.so.1",
+                       "libfontconfig.so.1",
+                       "libexpat.so.0",
+                      ]:
+            fname = os.path.join(dirname, fname0)
+            if os.path.exists(fname):
+                cmd =  "cp -f %s %s" % (fname, os.path.join(install_dir, fname0))
                 os.system(cmd)
-                fname = '/usr/lib/libpng.so.3'
-                cmd = "cp -f %s %s" % (fname,
-                                       os.path.join(install_dir, 'libpng.so.3'))
-                os.system(cmd)
-
         #numpy is now compiled with libgfortran at the ESRF
         for fname in glob.glob(os.path.join(dirname, "libgfortra*")):
-            cmd = "cp -f %s %s" % (fname,
+            if os.path.exists(fname):
+                cmd = "cp -f %s %s" % (fname,
                                     os.path.join(install_dir, os.path.basename(fname)))
-            os.system(cmd)
+                os.system(cmd)
 
     #remove libX of the packaging system to use that of the target system
-    for fname in glob.glob(os.path.join(install_dir,"libX*")):
-        os.remove(fname)
+    #for fname in glob.glob(os.path.join(install_dir,"libX*")):
+    #    os.remove(fname)
 
     #remove libfontconfig.so of the package in order to use the one in the target system
-    for fname in glob.glob(os.path.join(install_dir,"libfontconf*")):
-        os.remove(fname)
+    #for fname in glob.glob(os.path.join(install_dir,"libfontconf*")):
+    #    os.remove(fname)
 
     #end linux binary
 
@@ -490,11 +482,11 @@ if not SCIPY:
 sys.exit(0)
 
 ########################## WHAT FOLLOWS IS UNUSED CODE ################
-#                                                                     #               
+#                                                                     #
 # I was using it to add undetected modules to library.zip             #
-#                                                                     #               
+#                                                                     #
 # It might be needed in the future. Code kept here as "backup".       #
-#                                                                     #               
+#                                                                     #
 #######################################################################
 
 #Add modules to library.zip
@@ -538,7 +530,7 @@ def addToZip(zf, path, zippath, full=False):
                 print("NOT ADDING", path)
 
 addToZip(zf, PyMcaDir, os.path.basename(PyMcaDir), full=False)
-    
+
 #if PY_COUNTER > PYC_COUNTER:
 #    print "WARNING: More .py files than  .pyc files. Check cx_setup.py"
 if PY_COUNTER < PYC_COUNTER:
